package org.openhab.io.gcal.authorization;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.GeneralSecurityException;
import java.util.Collections;

import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.auth.oauth2.TokenResponseException;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeRequestUrl;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeTokenRequest;
import com.google.api.client.googleapis.auth.oauth2.GoogleClientSecrets;
import com.google.api.client.googleapis.auth.oauth2.GoogleTokenResponse;
import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.store.DataStoreFactory;
import com.google.api.client.util.store.FileDataStoreFactory;
import com.google.api.services.calendar.CalendarScopes;

public class Cli {
	/**
	   * Be sure to specify the name of your application. If the application name is {@code null} or
	   * blank, the application will log a warning. Suggested format is "MyCompany-ProductName/1.0".
	   */
	  private static final String APPLICATION_NAME = "";

	  /** Directory to store user credentials. */
	  private static final java.io.File DATA_STORE_DIR =
	      new java.io.File(System.getProperty("user.home"), ".store/gcal");

	  /**
	   * Global instance of the {@link DataStoreFactory}. The best practice is to make it a single
	   * globally shared instance across your application.
	   */
	  private static FileDataStoreFactory dataStoreFactory;
	 
	  /** Global instance of the HTTP transport. */
	  private static HttpTransport httpTransport;

	  /** Global instance of the JSON factory. */
	  private static final JsonFactory JSON_FACTORY = JacksonFactory.getDefaultInstance();

	  
	  
	  
	  
	public static void main(String[] args) {
		

		System.out.println("Gcal healper Version 1.0");
		
		if(args.length < 2)
		{
			printUsageMessage();
			return;
		}
		
		try {
			httpTransport = GoogleNetHttpTransport.newTrustedTransport();
			dataStoreFactory = new FileDataStoreFactory(DATA_STORE_DIR);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (GeneralSecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		String command = args[0].trim();
		String jsonFilePath = args[1];
		
		if(command.equalsIgnoreCase("getAuthURL"))
		{
			
			try {
				GoogleClientSecrets sec = getClientSecrets(jsonFilePath);
				GoogleAuthorizationCodeFlow fl = getGoogleAuthorizationCodeFlow(sec);
				
				String redirectURLs = sec.getInstalled().getRedirectUris().get(0);
		     	GoogleAuthorizationCodeRequestUrl url = fl.newAuthorizationUrl();
		    	String myURL = url.toString();
		    	myURL=myURL.replace("redirect_uri", "redirect_uri="+redirectURLs);
		    	printAuthURL(myURL);
				
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		}
		else if(command.equalsIgnoreCase("getUserCredentials"))
		{
			
			if(args.length<3)
			{
				System.out.println("ERROR: Third paraleter (code) is missing");
				return;
			}
			
			String code = args[2];
			
			try {
				GoogleClientSecrets sec = getClientSecrets(jsonFilePath);
				String redirectURLs = sec.getInstalled().getRedirectUris().get(0);
				GoogleAuthorizationCodeFlow fl = getGoogleAuthorizationCodeFlow(sec);
				GoogleAuthorizationCodeTokenRequest tr = fl.newTokenRequest(code);
				tr.setRedirectUri(redirectURLs);
		    	GoogleTokenResponse resp = tr.execute();
		    	Credential cred = fl.createAndStoreCredential(resp, "user1");
		    	
		    	System.out.println();
		    	System.out.println();
		    	System.out.println("Credentials stored sucessfully at: "+DATA_STORE_DIR.getCanonicalPath());
				
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		else
		{
			System.out.println("ERROR: unknown command: "+args[0]);
		}
	
	
	
		
		
		
		
		
		
	}
	
	private static void printUsageMessage()
	{
		
		System.out.println("Usage:");
		System.out.println("getAuthURL <full path to *.json secret file>");
		System.out.println("          (to get the google auth URL)");
		System.out.println("Or");
		System.out.println("getUserCredentials <full path to *.json secret file> <code>");
		System.out.println("          (to get the credentials)");
		System.out.println();
		System.out.println("For example:");
		System.out.println();
		System.out.println(">gcalHealper getAuthURL c:/gcal/client_secret.json");
		System.out.println();
		System.out.println("Or");
		System.out.println(">gcalHealper getUserCredentials c:/gcal/client_secret.json hijdldofd454857483dwxdk490489");
		
		
		
	}
	
	private static void printAuthURL(String url)
	{
		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println("Please open the following URL in a browser and complete the authorization flow:");
		System.out.println();
		System.out.println(url);
	}
	
	private static GoogleClientSecrets getClientSecrets(String jsonFilePath) throws FileNotFoundException, IOException
	{
		GoogleClientSecrets clientSecrets = GoogleClientSecrets.load(JSON_FACTORY,new InputStreamReader(new FileInputStream(jsonFilePath)));
		return clientSecrets;
	}
	
	private static GoogleAuthorizationCodeFlow getGoogleAuthorizationCodeFlow(GoogleClientSecrets clientSecrets) throws IOException
	{
		 // set up authorization code flow
	    GoogleAuthorizationCodeFlow flow = new GoogleAuthorizationCodeFlow.Builder(httpTransport, JSON_FACTORY, clientSecrets,Collections.singleton(CalendarScopes.CALENDAR)).setDataStoreFactory(dataStoreFactory).setAccessType("offline")
	        .build();
	    return flow;
	}
	
	
	

}
